==> mule-datasense-4.0.0-sources.jar: org/mule/datasense/impl/phases/builder/SimpleDependenciesResolutionScopeStrategy.java <==
package org.mule.datasense.impl.phases.builder;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.Multimap;
import org.mule.datasense.api.metadataprovider.ApplicationModel;
import org.mule.datasense.api.metadataprovider.DataSenseProvider;
import org.mule.datasense.impl.DefaultDataSense;
import org.mule.datasense.impl.util.ComponentIdentifierUtils;
import org.mule.runtime.api.component.ComponentIdentifier;
import org.mule.runtime.config.internal.model.ComponentModel;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Deque;
import java.util.Optional;

import static java.util.Optional.ofNullable;

public class SimpleDependenciesResolutionScopeStrategy extends DependenciesResolutionScopeStrategy {

  private ArrayList included;

  public SimpleDependenciesResolutionScopeStrategy(ApplicationModel applicationModel,
                                                   DataSenseProvider dataSenseProvider,
                                                   DataSenseResolutionScopeStrategy dataSenseResolutionScopeStrategy) {
    super(applicationModel, dataSenseProvider, dataSenseResolutionScopeStrategy);
    included = new ArrayList();
    init();
  }

  private void init() {
    final org.mule.runtime.config.internal.model.ApplicationModel muleApplicationModel =
        getApplicationModel().getMuleApplicationModel();

    Multimap<String, String> flowDependenciesByFlow = ArrayListMultimap.create();
    muleApplicationModel.executeOnEveryFlow(componentModel -> {
      final String flowName = componentModel.getNameAttribute();
      if (flowName != null) {
        componentModel.executedOnEveryInnerComponent(childComponentModel -> {
          final ComponentIdentifier componentIdentifier =
              ComponentIdentifierUtils.createFromComponentModel(childComponentModel);
          if (DefaultDataSense.COMPONENT_IDENTIFIER_FLOW_REF.equals(componentIdentifier)) {
            ofNullable(childComponentModel.getNameAttribute()).ifPresent(flowRefName -> {
              flowDependenciesByFlow.put(flowName, flowRefName);
            });
          }
        });
      }
    });

    Deque<String> deque = new ArrayDeque<>();

    // prefill queue with accepted flows
    muleApplicationModel.executeOnEveryFlow(componentModel -> {
      if (getDataSenseResolutionScopeStrategy().match(componentModel)) {
        ofNullable(componentModel.getNameAttribute()).ifPresent(deque::add);
      }
    });

    included.clear();
    while (!deque.isEmpty()) {
      final String flow = deque.pop();
      if (!included.contains(flow)) {
        included.add(flow);
        final Collection<String> flowDependencies = flowDependenciesByFlow.get(flow);
        flowDependencies.forEach(deque::offer);
      }
    }
  }

  @Override
  public boolean match(ComponentModel componentModel) {
    return super.match(componentModel) || visited(componentModel);
  }

  private boolean visited(ComponentModel componentModel) {
    return included.contains(componentModel.getNameAttribute());
  }
}

==> mule-datasense-4.1.6-sources.jar: org/mule/datasense/impl/phases/builder/SimpleDependenciesResolutionScopeStrategy.java <==
package org.mule.datasense.impl.phases.builder;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.Multimap;
import org.mule.datasense.api.metadataprovider.ApplicationModel;
import org.mule.datasense.api.metadataprovider.DataSenseProvider;
import org.mule.datasense.impl.DefaultDataSense;
import org.mule.datasense.impl.phases.typing.resolver.AggregatorListenerTypeResolver;
import org.mule.datasense.impl.util.ComponentIdentifierUtils;
import org.mule.runtime.api.component.ComponentIdentifier;
import org.mule.runtime.api.component.location.ComponentLocation;
import org.mule.runtime.config.internal.model.ComponentModel;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Deque;
import java.util.HashMap;
import java.util.Map;

import static java.util.Optional.ofNullable;

public class SimpleDependenciesResolutionScopeStrategy extends DependenciesResolutionScopeStrategy {

  private ArrayList<String> included;

  public SimpleDependenciesResolutionScopeStrategy(ApplicationModel applicationModel,
                                                   DataSenseProvider dataSenseProvider,
                                                   DataSenseResolutionScopeStrategy dataSenseResolutionScopeStrategy) {
    super(applicationModel, dataSenseProvider, dataSenseResolutionScopeStrategy);
    included = new ArrayList<>();
    init();
  }

  private void init() {
    final org.mule.runtime.config.internal.model.ApplicationModel muleApplicationModel =
        getApplicationModel().getMuleApplicationModel();

    Multimap<String, String> flowDependenciesByFlow = ArrayListMultimap.create();

    Map<String, ComponentLocation> componentLocationByNameMap = new HashMap<>();
    muleApplicationModel.executeOnEveryRootElement(componentModel -> {
      final String flowName = componentModel.getNameAttribute();
      if (flowName != null) {
        componentModel.executedOnEveryInnerComponent(childComponentModel -> {
          final String name = childComponentModel.getNameAttribute();
          componentLocationByNameMap.put(name, childComponentModel.getComponentLocation());
        });
      }
    });

    muleApplicationModel.executeOnEveryRootElement(componentModel -> {
      final String flowName = componentModel.getNameAttribute();
      if (flowName != null) {
        componentModel.executedOnEveryInnerComponent(childComponentModel -> {
          final ComponentIdentifier componentIdentifier =
              ComponentIdentifierUtils.createFromComponentModel(childComponentModel);
          if (DefaultDataSense.COMPONENT_IDENTIFIER_FLOW_REF.equals(componentIdentifier)) {
            ofNullable(childComponentModel.getNameAttribute()).ifPresent(flowRefName -> {
              flowDependenciesByFlow.put(flowName, flowRefName);
            });
          } else {
            final Map<String, String> parameters = childComponentModel.getParameters();
            final String componentRef = parameters.get(AggregatorListenerTypeResolver.PARAMETER_AGGREGATOR_REF);
            if (componentRef != null) {
              final ComponentLocation componentLocation = componentLocationByNameMap.get(componentRef);
              final String rootContainerName = componentLocation.getRootContainerName();
              flowDependenciesByFlow.put(flowName, rootContainerName);
            }
          }
        });
      }
    });

    Deque<String> deque = new ArrayDeque<>();

    // prefill queue with accepted flows
    muleApplicationModel.executeOnEveryRootElement(componentModel -> {
      if (getDataSenseResolutionScopeStrategy().match(componentModel)) {
        ofNullable(componentModel.getNameAttribute()).ifPresent(deque::add);
      }
    });

    included.clear();
    while (!deque.isEmpty()) {
      final String flow = deque.pop();
      if (!included.contains(flow)) {
        included.add(flow);
        final Collection<String> flowDependencies = flowDependenciesByFlow.get(flow);
        flowDependencies.forEach(deque::offer);
      }
    }
  }

  @Override
  public boolean match(ComponentModel componentModel) {
    return super.match(componentModel) || visited(componentModel);
  }

  private boolean visited(ComponentModel componentModel) {
    return included.contains(componentModel.getNameAttribute());
  }
}
